@@ -15,14 +15,14 @@ using namespace Rcpp;
 using namespace Eigen;
 using Rcpp::as;
 
-// //' @export
+// // ' @export
 //[[Rcpp::export]]
 Rcpp::List LU_dense_cpp(Eigen::Map<Eigen::MatrixXd> X_, Eigen::VectorXd & z_, Eigen::VectorXd & icoef_,
                         Eigen::ArrayXd & gsize_,Eigen::ArrayXd & pen_,
                         Eigen::ArrayXd & lambdaseq_,bool user_lambdaseq_,
                         int pathLength_,double lambdaMinRatio_, double pi_,
                         int maxit_,double tol_,double inner_tol_,
-                        bool useStrongSet_, bool verbose_, double stepSize_,double stepSizeAdj_,int batchSize_,
+                        bool useStrongSet_, bool verbose_, double stepSize_,double stepSizeAdj_,int batchSize_,int updateFreq_,
                         std::vector<double> samplingProbabilities_,bool useLipschitz_,std::string method_,int trace_)
 {
   
@@ -51,7 +51,7 @@ Rcpp::List LU_dense_cpp(Eigen::Map<Eigen::MatrixXd> X_, Eigen::VectorXd & z_, Ei
     }else{
       pgLUfit<Eigen::Map<MatrixXd> > lu(X_,z_,icoef_,gsize_,pen_,lambdaseq_,user_lambdaseq_,pathLength_,
                                         lambdaMinRatio_,pi_,maxit_,tol_,verbose_,
-                                        stepSize_,stepSizeAdj_,batchSize_,samplingProbabilities_,useLipschitz_,method_,trace_);
+                                        stepSize_,stepSizeAdj_,batchSize_,updateFreq_,samplingProbabilities_,useLipschitz_,method_,trace_);
       lu.pgLUfit_main();
       return Rcpp::List::create(Rcpp::Named("coef") = lu.getCoefficients(),
                                 Rcpp::Named("std_coef") = lu.getStdCoefficients(),
@@ -76,14 +76,14 @@ Rcpp::List LU_dense_cpp(Eigen::Map<Eigen::MatrixXd> X_, Eigen::VectorXd & z_, Ei
   return R_NilValue;
 }
 
-// //' @export
+// // ' @export
 //[[Rcpp::export]]
 Rcpp::List LU_sparse_cpp(Eigen::SparseMatrix<double> & X_, Eigen::VectorXd & z_, Eigen::VectorXd & icoef_,
                          Eigen::ArrayXd & gsize_,Eigen::ArrayXd & pen_,
                          Eigen::ArrayXd & lambdaseq_,bool user_lambdaseq_,
                          int pathLength_,double lambdaMinRatio_, double pi_,
                          int maxit_,double tol_,double inner_tol_,
-                         bool useStrongSet_, bool verbose_,double stepSize_,double stepSizeAdj_,int batchSize_,
+                         bool useStrongSet_, bool verbose_,double stepSize_,double stepSizeAdj_,int batchSize_,int updateFreq_,
                          std::vector<double> samplingProbabilities_,bool useLipschitz_,std::string method_,int trace_)
 {
   try{
@@ -111,7 +111,7 @@ Rcpp::List LU_sparse_cpp(Eigen::SparseMatrix<double> & X_, Eigen::VectorXd & z_,
     }else{
       pgLUfit<Eigen::SparseMatrix<double> > lu(X_,z_,icoef_,gsize_,pen_,lambdaseq_,user_lambdaseq_,pathLength_,
                                         lambdaMinRatio_,pi_,maxit_,tol_,verbose_,
-                                        stepSize_,stepSizeAdj_,batchSize_,samplingProbabilities_,useLipschitz_,method_,trace_);
+                                        stepSize_,stepSizeAdj_,batchSize_,updateFreq_,samplingProbabilities_,useLipschitz_,method_,trace_);
       lu.pgLUfit_main();
       return Rcpp::List::create(Rcpp::Named("coef") = lu.getCoefficients(),
                                 Rcpp::Named("std_coef") = lu.getStdCoefficients(),
